#ifndef __TEST_CLIENT_TESTS_H__
#define __TEST_CLIENT_TESTS_H__

#include <QDebug>
#include <QTest>
#include <QProcessEnvironment>

#include <hws.h>

namespace test
{
    //------------------------------------------------------------------------
    // Helper Object
    //------------------------------------------------------------------------

    class ClientTests;

    // This basically handles setting up slots that are not read in as
    // test functions.
    class ClientHelper : public QObject
    {
    Q_OBJECT

    public slots:

        void logInDone(hws::RequestErrorPtr error,
                       hws::SessionPtr session);


        void executeMethodDone(hws::RequestErrorPtr error,
                               const QString &method,
                               const QString &path,
                                const QSharedPointer<hws::Client::QVariantMapList> data);

    public:
        ClientTests *mTests;
    };

    // These are most of the 'integrated' tests in the system.
    class ClientTests : public QObject
    {
    Q_OBJECT

    private slots:
        //--------------------------------------------------------------------
        // QTest Framework
        //--------------------------------------------------------------------

        void initTestCase();

        void init();

        //--------------------------------------------------------------------
        // Test Methods
        //--------------------------------------------------------------------

        void logIn();

        void logInError();

        void listDepots();

        void listProjects();

        //--------------------------------------------------------------------
        // Test Helpers and State
        //--------------------------------------------------------------------

    private:
        void initClient();

        void waitForLogIn();

        void waitForExecuteMethod();

    public:
        ClientHelper mHelper;
        hws::ClientPtr mClient;
        hws::SessionPtr mSession;
        hws::RequestErrorPtr mRequestError;
        QString mMethod;
        QString mPath;
        QSharedPointer<hws::Client::QVariantMapList> mData;
        bool mLogInDone;
        bool mExecuteMethodDone;
    };

}

#endif // __TEST_CLIENT_TESTS_H__
